library(ggplot2)
library(plyr)
library(RColorBrewer)

nicknames <- new.env(parent=emptyenv())
nicknames[["109.105.109.165"]]  <- "ndnop3"
nicknames[["109.105.109.146"]]  <- "ndnop4"
nicknames[["109.105.109.147"]]  <- "ndnop5"
nicknames[["178.209.52.110"]]   <- "schanenlied"
nicknames[["83.212.101.3"]]     <- "LeifEricson"
nicknames[["104.131.108.182"]]  <- "mercurius4"
nicknames[["192.240.101.106"]]  <- "fdctorbridge01"
nicknames[["198.245.60.50"]]    <- "riemann"
nicknames[["192.99.11.54"]]     <- "noether"
nicknames[["154.35.22.9"]]      <- "GreenBelt"
nicknames[["154.35.22.10"]]     <- "Mosaddegh"
nicknames[["154.35.22.11"]]     <- "MaBishomarim"
nicknames[["154.35.22.12"]]     <- "JonbesheSabz"
nicknames[["154.35.22.13"]]     <- "Azadi"
nicknames[["192.30.252.130"]]   <- "github.com"
nicknames[["192.95.36.142"]]	<- "Lisbeth"
nicknames[["85.17.30.79"]]	<- "NX01"

# Return a matrix with columns "begin" and "end" that represents runs
# of data in v that are separated by no more than tolerance.
collapse_runs <- function(v, tolerance) {
	d <- diff(v) > tolerance
	cbind(begin=v[c(TRUE, d)], end=v[c(d, TRUE)])
}

x <- read.table("probe.csv", sep=",",
	col.names=c("date", "site", "host", "port", "elapsed", "success", "errno", "errmsg"),
	colClasses=c("numeric", "factor", "character", "integer", "numeric", "logical", "integer", "character"),
	na.strings = "None")
x$date <- as.POSIXct(x$date, origin="1970-01-01", tz="UTC")
x$site <- factor(x$site, levels=unique(c("eecs-login", "bauxite", "mistletoe", "spline", "castor", "muskie", "titan", "leopard", "gala", as.character(x$site))))

# Omit some test reports.
x <- x[x$site != "leopard-test", ]

x$label <- as.factor(sprintf("%s:%d", mget(x$host, envir=nicknames, ifnotfound=x$host), x$port))
x$label <- factor(x$label, levels=unique(x$label[order(x$host, x$port)]))

x$result <- factor(ifelse(x$success, "success", x$errmsg))
# These two results are most common so put them first.
x$result <- relevel(x$result, "timed out")
x$result <- relevel(x$result, "success")
pal <- c("#0000aa", "#cccccc", brewer.pal(nlevels(x$result)-2, "Set1"))

x <- x[order(x$date), ]

for (lab in levels(x$label)) {
	control <- x[x$site=="eecs-login" & x$label==lab, ]
	x$nearest.control[x$label==lab] <- row.names(control)[findInterval(x[x$label==lab, ]$date, control$date)]
}

x$control.result <- x[x$nearest.control, "result"]

tmp <- ddply(x, c("site"), summarise, date=collapse_runs(date, 40*60))
df <- data.frame(site=tmp$site, tmp$date)
df$begin <- as.POSIXct(df$begin, origin="1970-01-01", tz="UTC")
df$end <- as.POSIXct(df$end, origin="1970-01-01", tz="UTC")
p <- ggplot(df)
p <- p + geom_linerange(aes(x=site, ymin=begin, ymax=end), size=5)
p <- p + coord_flip()
p <- p + theme_bw()
ggsave("timespans.png", width=10, height=6, dpi=120)

x <- x[x$site%in%c("eecs-login", "muskie", "titan", "gala"), ]
p <- ggplot(x, aes(date, label, size=(result==control.result)))
p <- p + geom_blank()
p <- p + geom_point(data=x, shape="|", alpha=0.8, aes(date, as.numeric(label)+ifelse(success,0.00,-0.00), color=result))
p <- p + scale_color_manual(values=pal)
p <- p + scale_size_manual(values=c(1.5, 0.3))
p <- p + scale_x_datetime(limits=c(as.POSIXct("2016-08-01"), NA))
p <- p + geom_text(data=aggregate(date ~ site + label, x, min), aes(x=date, y=label, label=label), size=2.5, hjust=1, nudge_x=-100000)
p <- p + facet_grid(site ~ .)
p <- p + theme_bw()
p <- p + theme(legend.position="top")
p <- p + guides(color=guide_legend(override.aes=list(alpha=1, size=2, shape=15)))
ggsave("probe.png", width=14, height=30, dpi=90)
